@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2022 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -10,6 +10,7 @@ import re
 
 from xpra.client.client_widget_base import ClientWidgetBase
 from xpra.client.window_backing_base import fire_paint_callbacks
+from xpra.client.scaling_parser import scaleup_value, scaledown_value
 from xpra.os_util import bytestostr, OSX, WIN32, is_Wayland
 from xpra.common import GRAVITY_STR
 from xpra.util import net_utf8, typedict, envbool, envint, WORKSPACE_UNSET, WORKSPACE_NAMES
@@ -25,6 +26,7 @@ metalog = Logger("metadata")
 geomlog = Logger("geometry")
 iconlog = Logger("icon")
 alphalog = Logger("alpha")
+scalinglog = Logger("scaling")
 
 
 SIMULATE_MOUSE_DOWN = envbool("XPRA_SIMULATE_MOUSE_DOWN", True)
@@ -53,8 +55,11 @@ class ClientWindowBase(ClientWidgetBase):
         self.group_leader = group_leader
         self._pos = (wx, wy)
         self._size = (ww, wh)
+        self._xscale = client.xscale
+        self._yscale = client.yscale
         self._client_properties = client_properties
         self._set_initial_position = metadata.boolget("set-initial-position", False)
+        self._requested_position = metadata.intpair("requested-position", None)
         self.size_constraints = typedict()
         self.geometry_hints = {}
         self.content_type = ""
@@ -86,6 +91,7 @@ class ClientWindowBase(ClientWidgetBase):
         self.init_window(metadata)
         self.setup_window(bw, bh)
         self.update_metadata(metadata)
+        self.finalize_window()
 
     def __repr__(self):
         return "ClientWindow(%s)" % self._id
@@ -95,7 +101,6 @@ class ClientWindowBase(ClientWidgetBase):
         self._metadata = typedict()
         # used for only sending focus events *after* the window is mapped:
         self._been_mapped = False
-        self._override_redirect_windows = []
         def wn(w):
             return WORKSPACE_NAMES.get(w, w)
         workspace = typedict(self._client_properties).intget("workspace", None)
@@ -116,29 +121,21 @@ class ClientWindowBase(ClientWidgetBase):
         self.window_gravity = OVERRIDE_GRAVITY or sc.intget("gravity", DEFAULT_GRAVITY)
         self.set_decorated(metadata.boolget("decorations", True))
 
+    def finalize_window(self):
+        pass
+
 
     def get_info(self):
         attributes = []
-        if self._fullscreen:
-            attributes.append("fullscreen")
-        if self._maximized:
-            attributes.append("maximized")
-        if self._above:
-            attributes.append("above")
-        if self._below:
-            attributes.append("below")
-        if self._shaded:
-            attributes.append("shaded")
-        if self._sticky:
-            attributes.append("sticky")
-        if self._skip_pager:
-            attributes.append("skip-pager")
-        if self._skip_taskbar:
-            attributes.append("skip-taskbar")
-        if self._iconified:
-            attributes.append("iconified")
-        if self._focused:
-            attributes.append("focused")
+        for attr in (
+            "fullscreen", "maximized",
+            "above", "below",
+            "shaded", "sticky",
+            "skip-pager", "skip-taskbar",
+            "iconified", "focused",
+            ):
+            if getattr(self, "_%s" % attr.replace("-", "_"), False):
+                attributes.append(attr)
         info = super().get_info()
         info.update({
             "override-redirect"     : self._override_redirect,
@@ -167,6 +164,41 @@ class ClientWindowBase(ClientWidgetBase):
         return None
 
 
+    ######################################################################
+    # screen scaling:
+    def fsx(self, v):
+        """ convert X coordinate from server to client """
+        return v*self._xscale
+    def fsy(self, v):
+        """ convert Y coordinate from server to client """
+        return v*self._yscale
+    def sx(self, v) -> int:
+        """ convert X coordinate from server to client """
+        return round(self.fsx(v))
+    def sy(self, v) -> int:
+        """ convert Y coordinate from server to client """
+        return round(self.fsy(v))
+    def srect(self, x, y, w, h):
+        """ convert rectangle coordinates from server to client """
+        return self.sx(x), self.sy(y), self.sx(w), self.sy(h)
+    def sp(self, x, y):
+        """ convert X,Y coordinates from server to client """
+        return self.sx(x), self.sy(y)
+
+    def cx(self, v) -> int:
+        """ convert X coordinate from client to server """
+        return round(v/self._xscale)
+    def cy(self, v) -> int:
+        """ convert Y coordinate from client to server """
+        return round(v/self._yscale)
+    def crect(self, x, y, w, h):
+        """ convert rectangle coordinates from client to server """
+        return self.cx(x), self.cy(y), self.cx(w), self.cy(h)
+    def cp(self, x, y):
+        """ convert X,Y coordinates from client to server """
+        return self.cx(x), self.cy(y)
+
+
     def new_backing(self, bw, bh):
         backing_class = self.get_backing_class()
         log("new_backing(%s, %s) backing_class=%s", bw, bh, backing_class)
@@ -184,7 +216,6 @@ class ClientWindowBase(ClientWidgetBase):
     def destroy(self):
         #ensure we clear reference to other windows:
         self.group_leader = None
-        self._override_redirect_windows = []
         self._metadata = {}
         if self._backing:
             self._backing.close()
@@ -276,31 +307,44 @@ class ClientWindowBase(ClientWidgetBase):
                 "server-machine"  : getattr(self._client, "_remote_hostname", None) or UNKNOWN_MACHINE,
                 "server-display"  : getattr(self._client, "_remote_display", None) or "<unknown display>",
                 }
+            def validhostname(value):
+                if value not in (
+                    "localhost",
+                    "localhost.localdomain",
+                    "",
+                    None):
+                    return None
+                return value
             def getvar(var):
                 #"hostname" is magic:
                 #we try harder to find a useful value to show:
                 if var in ("hostname", "hostinfo"):
-                    if var=="hostinfo" and getattr(self._client, "mmap_enabled", False):
-                        #this is a local connection for sure
-                        server_display = getattr(self._client, "server_display", None)
-                        if server_display:
-                            return server_display
+                    server_display = getattr(self._client, "server_display", None)
+                    if var=="hostinfo" and getattr(self._client, "mmap_enabled", False) and server_display:
+                        #this is a local connection for sure and we can specify the display directly:
+                        return server_display
+                    import socket
+                    local_hostname = validhostname(socket.gethostname())
                     #try to find the hostname:
                     proto = getattr(self._client, "_protocol", None)
+                    target = None
                     if proto:
                         conn = getattr(proto, "_conn", None)
                         if conn:
-                            hostname = conn.info.get("host") or bytestostr(conn.target)
+                            hostname = conn.info.get("host")
+                            target = bytestostr(conn.target)
                             if hostname:
+                                if local_hostname==hostname and server_display:
+                                    return server_display
                                 return hostname
                     for m in ("client-machine", "server-machine"):
-                        value = getvar(m)
-                        if value not in (
-                            "localhost",
-                            "localhost.localdomain",
-                            "",
-                            None):
+                        value = validhostname(getvar(m))
+                        if value and value!=local_hostname:
                             return value
+                    if server_display:
+                        return server_display
+                    if target:
+                        return target
                     return UNKNOWN_MACHINE
                 value = metadata.get(var) or self._metadata.get(var)
                 if value is None:
@@ -573,7 +617,7 @@ class ClientWindowBase(ClientWidgetBase):
                 v = size_constraints.intpair(a)
                 if v:
                     v1, v2 = v
-                    hints[h] = (v1*self._client.xscale)/(v2*self._client.yscale)
+                    hints[h] = (v1*self._xscale)/(v2*self._yscale)
         #apply max-size override if needed:
         w,h = max_window_size
         if w>0 and h>0 and not self._fullscreen:
@@ -599,7 +643,7 @@ class ClientWindowBase(ClientWidgetBase):
             #not honouring "base" + "inc", but honouring just "min" instead:
             maxw = max(minw, maxw)
             maxh = max(minh, maxh)
-            scaling_info = self._client.xscale, self._client.yscale
+            scaling_info = self._xscale, self._yscale
             geomlog("modified hints for max window size %s and scaling %s: %s (rw=%s, rh=%s) -> max=%sx%s",
                     max_window_size, scaling_info, hints, rw, rh, maxw, maxh)
             #ensure we don't have duplicates with bytes / strings,
@@ -698,18 +742,47 @@ class ClientWindowBase(ClientWidgetBase):
             self._client.send_min_speed()
             log("new min-speed=%s", self._client.min_speed)
 
+
     def scaleup(self, *_args):
         self._client.scaleup()
 
+    def window_scaleup(self, *_args):
+        scaling = max(self._xscale, self._yscale)
+        options = scaleup_value(scaling)
+        scalinglog("scaleup() options=%s", options)
+        if options:
+            self._scaleto(min(options))
+
     def scaledown(self, *_args):
         self._client.scaledown()
 
+    def window_scaledown(self, *_args):
+        scaling = min(self._xscale, self._yscale)
+        options = scaledown_value(scaling)
+        scalinglog("scaledown() options=%s", options)
+        if options:
+            self._scaleto(max(options))
+
     def scalingoff(self):
         self._client.scalingoff()
 
+    def window_scalingoff(self):
+        self._scaleto(1)
+
+    def _scaleto(self, scaling):
+        xscale = self._xscale
+        yscale = self._yscale
+        w, h = self._size
+        self._xscale = self._yscale = scaling
+        nw = round(w/xscale*self._xscale)
+        nh = round(h/yscale*self._yscale)
+        self.reset_size_constraints()
+        self.resize(nw, nh)
+
     def scalereset(self, *_args):
         self._client.scalereset()
 
+
     def magic_key(self, *args):
         b = self.border
         if b:
@@ -755,7 +828,7 @@ class ClientWindowBase(ClientWidgetBase):
         backing = self._backing
         if not backing:
             return
-        if backing.repaint_all or self._client.xscale!=1 or self._client.yscale!=1 or is_Wayland():
+        if backing.repaint_all or self._xscale!=1 or self._yscale!=1 or is_Wayland():
             #easy: just repaint the whole window:
             rw, rh = self.get_size()
             self.idle_add(self.repaint, 0, 0, rw, rh)
@@ -796,14 +869,10 @@ class ClientWindowBase(ClientWidgetBase):
 
 
     def _focus(self):
-        focuslog("_focus() wid=%s, focused=%s", self._id, self._client._focused)
-        if self._client._focused!=self._id:
-            self._client.update_focus(self._id, True)
+        return self._client.update_focus(self._id, True)
 
     def _unfocus(self):
-        focuslog("_unfocus() wid=%s, focused=%s", self._id, self._client._focused)
-        if self._client._focused==self._id:
-            self._client.update_focus(self._id, False)
+        return self._client.update_focus(self._id, False)
 
     def quit(self):
         self._client.quit(0)
@@ -834,6 +903,12 @@ class ClientWindowBase(ClientWidgetBase):
     def show_file_upload(self, *args):
         self._client.show_file_upload(*args)
 
+    def show_shortcuts(self, *args):
+        self._client.show_shortcuts(*args)
+
+    def show_docs(self, *args):
+        self._client.show_docs(*args)
+
 
     def log(self, message=""):
         log.info(message)
