@@ -19,6 +19,7 @@ from gi.repository import GLib, Gdk, Gtk  #pylint: disable=no-name-in-module
 
 from xpra.util import flatten_dict, envbool
 from xpra.os_util import load_binary_file
+from xpra.common import FULL_INFO
 from xpra.gtk_common.gobject_compat import register_os_signals
 from xpra.server import server_features
 from xpra.server.server_base import ServerBase
@@ -56,7 +57,17 @@ class GTKServerBase(ServerBase):
         ### Set up keymap change notification:
         display = Gdk.Display.get_default()
         keymap = Gdk.Keymap.get_for_display(display)
-        keymap.connect("keys-changed", self._keys_changed)
+        #this event can fire many times in succession
+        #throttle how many times we call self._keys_changed()
+        def keys_changed(*_args):
+            if self.keymap_changing:
+                return
+            self.keymap_changing = True
+            def do_keys_changed():
+                self._keys_changed()
+                self.keymap_changing = False
+            self.timeout_add(500, do_keys_changed)
+        keymap.connect("keys-changed", keys_changed)
 
     def install_signal_handlers(self, callback):
         sstr = "%s server" % self.get_server_mode()
@@ -109,12 +120,11 @@ class GTKServerBase(ServerBase):
             self.ui_watcher.start()
         if server_features.windows:
             display = Gdk.Display.get_default()
-            i=0
-            while i<display.get_n_screens():
-                screen = display.get_screen(i)
-                screen.connect("size-changed", self._screen_size_changed)
-                screen.connect("monitors-changed", self._monitors_changed)
-                i += 1
+            #n = display.get_n_screens()
+            #assert n==1, "unsupported number of screens: %i" % n
+            screen = display.get_default_screen()
+            screen.connect("size-changed", self._screen_size_changed)
+            screen.connect("monitors-changed", self._monitors_changed)
         log("do_run() calling %s", Gtk.main)
         Gtk.main()
         log("do_run() end of gtk.main()")
@@ -122,7 +132,7 @@ class GTKServerBase(ServerBase):
 
     def make_hello(self, source):
         capabilities = super().make_hello(source)
-        if source.wants_display:
+        if "display" in source.wants:
             display = Gdk.Display.get_default()
             max_size = tuple(display.get_maximal_cursor_size())
             capabilities.update({
@@ -130,7 +140,7 @@ class GTKServerBase(ServerBase):
                 "cursor.default_size"   : display.get_default_cursor_size(),
                 "cursor.max_size"       : max_size,
                 })
-        if source.wants_versions:
+        if "versions" in source.wants and FULL_INFO>2:
             capabilities.update(flatten_dict(get_gtk_version_info()))
         return capabilities
 
